"""Module containing Enums for jitter symbols.

Contains:
- Symbols from the Plotly graphing library
https://plotly.com/python/marker-style/#custom-marker-symbols
"""

from enum import Enum


class PlotSymbols(Enum):
    """
    From theplotly graphing library: https://plotly.com/python/marker-style/#custom-marker-symbols
    """

    CIRCLE = "circle"
    CROSS = "cross"
    DIAMOND = "diamond"
    HEXAGRAM = "hexagram"



"""dashboard_container"""

from dash import html


def dashboard_container(children):
    """
    A HTML wrapper for a whole dashboard.

    Applies styles that make sure the application uses the full width of the browser.

    Unfortunately, due to the way Dash adds further HTML wrappers when it serves content,
    this wrapper cannot be applied once in template.html,
    and must be added within the application for each new dashboard.
    """
    return html.Div(children, className="dashboard-container")



"""Function to update the background colour and margin for plots"""

import plotly.graph_objects as go
from gov_uk_dashboards import colours


def update_layout_bgcolor_margin(fig: go.Figure, colour: str):
    """update background colour and margin for plot"""
    fig.update_layout(
        plot_bgcolor=colour,
        paper_bgcolor=colour,
        yaxis_zerolinecolor=colour,
        margin={"l": 0, "r": 0, "b": 0, "t": 0},
    )
    fig.update_xaxes(
        gridcolor=colour,
        zerolinecolor=colour,
        ticks="outside",
        tickcolor=colours.GovUKColours.MID_GREY.value,
    )
    fig.update_yaxes(
        gridcolor=colours.GovUKColours.MID_GREY.value,
        zerolinecolor=colours.GovUKColours.MID_GREY.value,
    )



"""Constants that are common across the dashboard."""

CUSTOM_DATA = "custom_data"
HOVER_TEXT_HEADERS = "hover_text_headers"
DATE_VALID = "Date valid"
CHART_LABEL_FONT_SIZE = 19
FINANCIAL_YEAR_ENDING = "Financial year ending"
VALUE = "Value"
MAIN_TITLE = "main_title"
SUBTITLE = "subtitle"
MEASURE = "measure"
FILL_TO_PREVIOUS_TRACE = "Fill to previous trace"
REMOVE_INITIAL_MARKER = "Remove initial marker"
YEAR = "Year"


UNIT_SIZE = "Unit size"
DEFAULT_COLOURSCALE = "tealgrn"

LEGEND_SPACING = "\u00a0" * 5

DOWNLOAD_BUTTON_CLASSES = (
    "govuk-button govuk-button--primary "
    "govuk-!-margin-bottom-0 govuk-!-margin-top-4 "
    "flex w-auto items-center gap-2 print:hidden"
)



"""format_visualisation_title"""

from dash import html


def format_visualisation_title(title):
    """Apply Gov.UK Design System styles to format title"""
    return html.Div(
        [
            html.Div(
                [
                    html.H1(
                        title,
                        id="title",
                        className="govuk-heading-m govuk-!-margin-bottom-1",
                    ),
                ],
                className="govuk-grid-column-full",
            )
        ],
        className="govuk-grid-row",
    )



"""Module containing functions related to graph axes.

Contains:
- calc_axis_range: Return a range for a column of a dataframe so the derived
    axis is at the origin or the lowest negative value in the column, whichever
    is lower.
"""

# from math import floor, ceil

# import pandas as pd


# def calc_axis_range(dataframe: pd.DataFrame, column: str) -> list:
#     """Show origin on dashboard axis or negative value for numeric type column"""
#     range_multiplier = 1.01
#     axis_range = [
#         floor(dataframe[[column]].min().iloc[0] * range_multiplier),
#         ceil(dataframe[[column]].max().iloc[0] * range_multiplier),
#     ]
#     if axis_range[0] < 0:
#         return axis_range
#     return [0, axis_range[1]]



"""main_content"""

from dash import html


def main_content(children):
    """
    Wrapper for the main content of the dashboard, containing visualisations.

    https://developer.mozilla.org/en-US/docs/Web/HTML/Element/main

    Using semantic HTML elements makes it easier for users with accessibility issues to
    interpret webpages using assistive devices.

    See https://developer.mozilla.org/en-US/docs/Learn/Accessibility/HTML for more guidance.
    """
    return html.Main(children, className="main", id="main-content", role="main")
